import os
import shutil
import sqlite3
import tempfile
import urllib.request
from collections import defaultdict
from dataclasses import replace
from logging import getLogger, basicConfig
from typing import Tuple, Iterator, Dict, List

import jusho.gateway as gateway
from .models import Address, Prefecture, City, TABLE_COUNT

ken_all_url = "https://www.post.japanpost.jp/zipcode/dl/kogaki/zip/ken_all.zip"
ken_all_rome_url = "https://www.post.japanpost.jp/zipcode/dl/roman/KEN_ALL_ROME.zip"

basicConfig(level="INFO")
logger = getLogger(__name__)


def tqdm(iterable: Iterator) -> Tuple[int, str]:
    values = list(iterable)
    length = len(values)
    for i, v in enumerate(values):
        yield i, v
        percent = i / length * 100
        if i % int(length / 100) == 0:
            print("\r{}% | {}/{}".format(int(percent), i + 1, length), end="")
    print("\r100% | {}/{}".format(length, length))


def han_to_zen(text: str) -> str:
    zenkaku = {
        i: v
        for i, v in enumerate(
            "、。「」　－０１２３４５６７８９"
            "アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヲン"
            "ッァィゥェォャュョ"
            "ガギグゲゴザジズゼゾダヂヅデドバビブベボヴ"
            "パピプペポ（）ーａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ／＜＞・．"
        )
    }

    hankaku = (
        "､｡｢｣ -0123456789"
        "ｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓﾔﾕﾖﾗﾘﾙﾚﾛﾜｦﾝ"
        "ｯｧｨｩｪｫｬｭｮ"
        "ｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾊﾋﾌﾍﾎｳ"
        "ﾊﾋﾌﾍﾎ()ｰabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ/<>･."
    )
    dakuten, handakuten = "ﾞﾟ"
    result = []
    i = 0
    max_index = len(text) - 1
    while i < len(text):
        c = text[i]
        index = hankaku.find(c)
        if index == -1:
            result.append(c)
            i += 1
        elif index == 18:
            if i != max_index and text[i + 1] == dakuten:
                result.append("ヴ")
                i += 2
            else:
                result.append("ウ")
                i += 1
        elif 21 <= index <= 35:  # ｶ-ｺ, ｻ-ｿ, ﾀ-ﾄ
            if i != max_index and text[i + 1] == dakuten:
                result.append(zenkaku[index + 50])
                i += 2
            else:
                result.append(zenkaku[index])
                i += 1
        elif 41 <= index <= 45:  # ﾊ-ﾎ
            if i != max_index and text[i + 1] == dakuten:
                result.append(zenkaku[index + 45])
                i += 2
            elif i != max_index and text[i + 1] == handakuten:
                result.append(zenkaku[index + 51])
                i += 2
            else:
                result.append(zenkaku[index])
                i += 1
        else:
            result.append(zenkaku[index])
            i += 1
    return "".join(result)


def insert_data(database_path: str, ken_all_path: str, ken_all_rome_path: str):
    prefectures: Dict[str, Prefecture] = {}
    cities: Dict[str, City] = {}
    addr: Dict[str, Dict[str, List[Address]]] = defaultdict(lambda: defaultdict(list))

    with open(ken_all_path, "r", encoding="shift-jis") as f:
        logger.info("parse ken_all.csv")
        for i, line in tqdm(f.read().splitlines()):
            (
                admin_division_code,
                old_zip_code,
                zip_code,
                prefecture_kana,
                city_kana,
                kana,
                prefecture_kanji,
                city_kanji,
                kanji,
                multiple_zip_code,
                multiple_address,
                has_chome,
                multiple_town_area,
                *_,
            ) = map(lambda x: x.strip('"'), line.split(","))

            old_zip_code = old_zip_code.strip()
            prefecture_kana = han_to_zen(prefecture_kana)
            city_kana = han_to_zen(city_kana)
            kana = han_to_zen(kana)

            if prefecture_kanji not in prefectures:
                prefectures[prefecture_kanji] = Prefecture(
                    len(prefectures) * TABLE_COUNT,
                    prefecture_kanji,
                    prefecture_kana,
                    "",
                )
            city_key = (
                (prefecture_kanji + city_kanji).replace("　", "").replace("\u3000", "")
            )
            if city_key not in cities:
                cities[city_key] = City(
                    len(cities) * TABLE_COUNT + 1,
                    prefectures[prefecture_kanji],
                    city_kanji,
                    city_kana,
                    "",
                )

            address = Address(
                id=i * TABLE_COUNT + 2,
                city=cities[city_key],
                admin_division_code=admin_division_code,
                old_zip_code=old_zip_code,
                zip_code=zip_code,
                kanji=kanji,
                kana=kana,
                eng="",
                multiple_zip_code=multiple_zip_code,
                multiple_address=multiple_address,
                has_chome=has_chome,
                multiple_town_area=multiple_town_area,
            )
            key = address.concat_kanji.replace("　", "").replace("\u3000", "")
            addr[address.prefecture.kanji][key].append(address)

    with open(ken_all_rome_path, "r", encoding="shift-jis") as f:
        logger.info("parse ken_all_rome.csv")
        for _, line in tqdm(f.read().splitlines()):
            (
                zipcode,
                prefecture_kanji,
                city_kanji,
                kanji,
                pref_eng,
                city_eng,
                town_eng,
            ) = map(lambda x: x.strip('"'), line.split(","))

            if (
                prefecture_kanji in prefectures
                and not prefectures[prefecture_kanji].eng
            ):
                prefectures[prefecture_kanji] = replace(
                    prefectures[prefecture_kanji], eng=pref_eng
                )

            city_key = (
                (prefecture_kanji + city_kanji).replace("　", "").replace("\u3000", "")
            )
            if city_key in cities and not cities[city_key].eng:
                cities[city_key] = replace(cities[city_key], eng=city_eng)

            key = f"{prefecture_kanji}{city_kanji}{kanji}".replace("　", "").replace(
                "\u3000", ""
            )
            if key in addr[prefecture_kanji]:
                ads = addr[prefecture_kanji][key]
            else:
                for k, v in addr[prefecture_kanji].items():
                    if k.startswith(key):
                        ads = v
                        break
                    if any(ad.zip_code == zipcode for ad in v):
                        ads = v
                        break
                else:
                    continue
            for i, ad in enumerate(ads):
                if not ad.eng:
                    ads[i] = replace(ad, eng=town_eng)

    if os.path.exists(database_path):
        os.remove(database_path)
    conn = sqlite3.connect(database_path)

    c = conn.cursor()
    c.executescript(gateway.insert.CREATE_TABLES)

    conn.commit()
    for pref in prefectures.values():
        gateway.insert.insert_prefecture(c, pref)
    for city in cities.values():
        gateway.insert.insert_city(c, city)
    for pref in addr.values():
        for ads in pref.values():
            for ad in ads:
                gateway.insert.insert_address(c, ad)

    conn.commit()
    conn.close()


def download_csv(output_dir):
    with tempfile.TemporaryDirectory() as tmp_dir:
        urllib.request.urlretrieve(ken_all_url, os.path.join(tmp_dir, "ken_all.zip"))
        urllib.request.urlretrieve(
            ken_all_rome_url, os.path.join(tmp_dir, "ken_all_rome.zip")
        )
        shutil.unpack_archive(os.path.join(tmp_dir, "ken_all.zip"), output_dir)
        shutil.unpack_archive(os.path.join(tmp_dir, "ken_all_rome.zip"), output_dir)


def create_database(database_path: str):
    if os.path.exists(database_path):
        os.remove(database_path)
    with tempfile.TemporaryDirectory() as tmp_dir:
        download_csv(tmp_dir)
        insert_data(
            database_path,
            os.path.join(tmp_dir, "ken_all.csv"),
            os.path.join(tmp_dir, "ken_all_rome.csv"),
        )



from dataclasses import replace
from sqlite3 import Cursor
from typing import List, Optional

from .utils import data_to_address
from ..models import Prefecture, City, Address


def fetch_address_by_zip_code(cursor: Cursor, zip_code: str) -> List[Address]:
    cursor.execute(
        "SELECT p.*, c.*, a.* FROM addresses AS a "
        "JOIN cities AS c ON c.id = a.city_id "
        "JOIN prefectures AS p ON p.id = c.prefecture_id "
        "WHERE a.zip_code=?",
        (zip_code,),
    )
    data = cursor.fetchall()

    return [data_to_address(v) for v in data]


def fetch_addresses_from_city(cursor: Cursor, city: City) -> List[Address]:
    cursor.execute("SELECT * FROM addresses " "WHERE city_id=?", (city.id,))
    data = cursor.fetchall()
    addresses = []
    for v in data:
        addresses.append(replace(Address(*v), city=city))

    return addresses


def fetch_address_by_id(cursor: Cursor, id_: int) -> Optional[Address]:
    cursor.execute(
        (
            "SELECT p.*, c.*, a.* FROM addresses AS a "
            "JOIN cities AS c ON c.id = a.city_id "
            "JOIN prefectures AS p ON p.id = c.prefecture_id "
            "WHERE a.id=?"
        ),
        (id_,),
    )
    data = cursor.fetchone()
    return data_to_address(data) if data else None


def search_addresses(
    cursor: Cursor, query: str, prefecture: Prefecture, city: City, type_: str
) -> List[Address]:
    query = [
        f"SELECT p.*, c.*, a.* FROM addresses AS a "
        f"JOIN cities AS c ON c.id = a.city_id "
        f"JOIN prefectures AS p ON p.id = c.prefecture_id "
        f"WHERE a.{type_} LIKE '%{query}%'",
        [],
    ]
    if prefecture:
        query[0] += " AND p.id = ?"
        query[1].append(prefecture.id)
    if city:
        query[0] += " AND c.id = ?"
        query[1].append(city.id)
    cursor.execute(*query)
    data = cursor.fetchall()

    return [data_to_address(v) for v in data]


__all__ = [
    "fetch_address_by_zip_code",
    "fetch_addresses_from_city",
    "fetch_address_by_id",
    "search_addresses",
]



from sqlite3 import Cursor
from typing import List, Optional

from .utils import data_to_city
from ..models import Prefecture, City
from dataclasses import replace


def fetch_cities_from_prefecture(cursor: Cursor, prefecture: Prefecture) -> List[City]:
    cursor.execute("SELECT * FROM cities WHERE prefecture_id=?", (prefecture.id,))
    data = cursor.fetchall()
    cities = []
    for v in data:
        cities.append(replace(City(*v), prefecture=prefecture))

    return cities


def fetch_city_by_id(cursor: Cursor, id_: int) -> Optional[City]:
    cursor.execute(
        (
            "SELECT p.*, c.* FROM cities AS c "
            "JOIN prefectures AS p ON p.id = c.prefecture_id "
            "WHERE c.id=?"
        ),
        (id_,),
    )
    data = cursor.fetchone()
    return data_to_city(data) if data else None


def search_cities(
    cursor: Cursor, query: str, prefecture: Prefecture, type_: str
) -> List[City]:
    query = [
        f"SELECT p.*, c.* FROM cities AS c "
        f"JOIN prefectures AS p ON p.id = c.prefecture_id "
        f"WHERE c.{type_} LIKE '%{query}%'",
        [],
    ]
    if prefecture:
        query[0] += " AND p.id = ?"
        query[1].append(prefecture.id)

    cursor.execute(*query)
    data = cursor.fetchall()

    return [data_to_city(v) for v in data]



import os
import re
import sqlite3
import sys
import warnings
from typing import List, Union

from jusho import gateway
from .models import Address, Prefecture, City, id_to_table_type


def get_database_path() -> str:
    file_paths = [
        os.path.join(os.path.dirname(__file__), "data/address.db"),
        os.path.join(sys.prefix, "data/address.db"),
        os.path.join(os.path.dirname(__file__).rsplit("lib", 1)[0], "data/address.db"),
    ]
    for path in file_paths:
        if os.path.exists(path):
            return path
    raise FileNotFoundError("database file not found")


class Jusho:
    def __init__(self, database_path=None):
        path = database_path or get_database_path()
        if not os.path.exists(path):
            raise FileNotFoundError(
                "database file not found. \n"
                "Consider to create a database file with jusho.create_database()"
            )
        self.conn = sqlite3.connect(database_path or get_database_path())
        self.cursor = self.conn.cursor()
        self.c = self.cursor

    def by_zip_code(self, zip_code: str) -> List[Address]:
        zip_code = "".join(re.findall(r"\d+", zip_code))
        return gateway.addresses.fetch_address_by_zip_code(self.cursor, zip_code)

    def fetch_by_id(self, id_: int) -> Union[Prefecture, City, Address]:
        table = id_to_table_type(id_)
        if table == "prefectures":
            return self.prefecture_by_id(id_)
        elif table == "cities":
            return self.city_by_id(id_)
        elif table == "addresses":
            return self.address_by_id(id_)
        else:
            raise ValueError(f"id_: {id_} does not seem to be a valid id")

    # #-#-#-#-#-#-#-#-# PREFECTURES #-#-#-#-#-#-#-#-# #
    def search_prefectures(self, query: str, type_="kanji") -> List[Prefecture]:
        return gateway.prefectures.search_prefectures(self.cursor, query, type_)

    @property
    def prefectures(self) -> List[Prefecture]:
        return gateway.prefectures.fetch_prefectures(self.cursor)

    def prefecture_by_id(self, id_: int) -> Prefecture:
        table = id_to_table_type(id_)
        if table != "prefectures":
            warnings.warn(
                f"id_: {id_} does not seem to be a prefecture id. rather, it's a {table} id",
                RuntimeWarning,
            )
        return gateway.prefectures.fetch_prefecture_by_id(self.cursor, id_)

    # #-#-#-#-#-#-#-#-# CITIES #-#-#-#-#-#-#-#-# #
    def cities(self, prefecture: Prefecture) -> List[City]:
        return gateway.cities.fetch_cities_from_prefecture(self.cursor, prefecture)

    def search_cities(
        self, query: str, prefecture: Prefecture = None, type_="kanji"
    ) -> List[City]:
        """
        search cities from query
        if prefecture is specified, search cities from prefecture
        """
        if type_ not in ("kanji", "kana", "eng"):
            raise ValueError("type must be one of 'kanji', 'kana', 'eng'")

        return gateway.cities.search_cities(self.cursor, query, prefecture, type_)

    def city_by_id(self, id_: int) -> City:
        table = id_to_table_type(id_)
        if table != "cities":
            warnings.warn(
                f"id_: {id_} does not seem to be a city id. rather, it's a {table} id",
                RuntimeWarning,
            )
        return gateway.cities.fetch_city_by_id(self.cursor, id_)

    # #-#-#-#-#-#-#-#-# ADDRESSES #-#-#-#-#-#-#-#-# #
    def addresses(self, city: City) -> List[Address]:
        return gateway.addresses.fetch_addresses_from_city(self.cursor, city)

    def search_addresses(
        self,
        query: str,
        prefecture: Prefecture = None,
        city: City = None,
        type_="kanji",
    ) -> List[Address]:
        """
        search addresses from query
        if prefecture is specified, search addresses from prefecture
        if city is specified, search addresses from city
        You can specify prefecture and city at the same time. But city must be a part of prefecture, and it's meaningless.
        """
        if type_ not in ("kanji", "kana", "eng"):
            raise ValueError("type must be one of 'kanji', 'kana', 'eng'")
        if prefecture and city:
            if city.prefecture != prefecture:
                raise ValueError("city must be a part of prefecture")

        return gateway.addresses.search_addresses(
            self.cursor, query, prefecture, city, type_
        )

    def address_by_id(self, id_: int) -> Address:
        table = id_to_table_type(id_)
        if table != "addresses":
            warnings.warn(
                f"id_: {id_} does not seem to be a address id. rather, it's a {table} id",
                RuntimeWarning,
            )
        return gateway.addresses.fetch_address_by_id(self.cursor, id_)

    def close(self):
        self.conn.close()

    def __del__(self):
        self.close()



from dataclasses import dataclass

TABLE_COUNT = 3


def id_to_table_type(id_: int) -> str:
    tables = ("prefectures", "cities", "addresses")
    return tables[id_ % TABLE_COUNT]


@dataclass(frozen=True)
class Prefecture:
    id: int  # mod TABLE_COUNT = 0
    kanji: str
    kana: str
    eng: str

    @property
    def to_sql(self):
        return f"{self.id}, '{self.kanji}', '{self.kana}', '{self.eng}'"


@dataclass(frozen=True)
class City:
    id: int  # mod TABLE_COUNT = 1

    prefecture: Prefecture
    kanji: str
    kana: str
    eng: str

    @property
    def concat_kanji(self) -> str:
        return f"{self.prefecture.kanji}　{self.kanji}"

    @property
    def concat_kana(self) -> str:
        return f"{self.prefecture.kana}　{self.kana}"

    @property
    def concat_eng(self) -> str:
        return f"{self.eng}, {self.prefecture.eng.title()}".title()


@dataclass(frozen=True)
class Address:
    id: int  # mod TABLE_COUNT = 2

    city: City

    admin_division_code: str
    old_zip_code: str
    zip_code: str

    kanji: str
    kana: str
    eng: str

    multiple_zip_code: bool
    multiple_address: bool
    has_chome: bool
    multiple_town_area: bool

    @property
    def concat_kanji(self) -> str:
        return f"{self.city.concat_kanji}　{self.kanji}"

    @property
    def concat_kana(self) -> str:
        return f"{self.city.concat_kana}　{self.kana}"

    @property
    def concat_eng(self) -> str:
        return f"{self.eng}, {self.city.concat_eng}".title()

    @property
    def prefecture(self) -> Prefecture:
        return self.city.prefecture

    @property
    def hyphen_zip(self):
        return "{}-{}".format(self.zip_code[:3], self.zip_code[3:])

    def __str__(self):
        if self.kanji == "以下に掲載がない場合":
            return "〒{}-{}, {} {}({} {})".format(
                self.zip_code[:3],
                self.zip_code[3:],
                self.prefecture.kanji,
                self.city.kanji,
                self.prefecture.eng,
                self.city.eng,
            )
        return "〒{}-{}, {} {} {}({} {} {})".format(
            self.zip_code[:3],
            self.zip_code[3:],
            self.prefecture.kanji,
            self.city.kanji,
            self.kanji,
            self.prefecture.eng,
            self.city.eng,
            self.eng,
        )



from collections import Counter

import pytest

from jusho import Jusho, Prefecture, City, Address


class TestJusho:
    """
    tests class of jusho
    """

    postman = Jusho()

    def test_by_zip_code(self):
        zip_codes = ["1600021", "160-0021", "160 0021", "〒160-0021"]
        for code in zip_codes:
            addresses = self.postman.by_zip_code(code)
            assert 1 == len(addresses)
            address = addresses[0]
            assert "160-0021" == address.hyphen_zip
            assert "歌舞伎町" == address.kanji
            assert "新宿区" == address.city.kanji
            assert "東京都" == address.prefecture.kanji
            assert "東京都　新宿区　歌舞伎町" == address.concat_kanji
            assert "トウキョウト　シンジュクク　カブキチョウ" == address.concat_kana
            assert "Kabukicho, Shinjuku Ku, Tokyo To" == address.concat_eng

    def test_prefectures(self):
        prefectures = self.postman.prefectures
        assert 47 == len(prefectures)
        lasts = Counter([p.kanji[-1] for p in prefectures])
        assert 1 == lasts["都"]
        assert 2 == lasts["府"]
        assert 1 == lasts["道"]

    def test_search_prefectures(self):
        prefectures = self.postman.search_prefectures("東京", "kanji")
        assert 1 == len(prefectures)
        assert "東京都" == prefectures[0].kanji

        prefectures = self.postman.search_prefectures("トウキョウト", "kana")
        assert 1 == len(prefectures)
        assert "トウキョウト" == prefectures[0].kana

        prefectures = self.postman.search_prefectures("tokyo", "eng")
        assert 1 == len(prefectures)
        assert "tokyo to" == prefectures[0].eng.lower()

    def test_cities(self):
        pref = self.postman.search_prefectures("東京", "kanji")[0]
        cities = self.postman.cities(pref)
        assert 62 == len(cities)
        assert pref == cities[0].prefecture

    def test_search_cities(self):
        tokyo = self.postman.search_prefectures("東京", "kanji")[0]
        osaka = self.postman.search_prefectures("大阪", "kanji")[0]
        cities = self.postman.search_cities("大阪市都島区")
        assert 1 == len(cities)
        assert osaka == cities[0].prefecture

        cities = self.postman.search_cities("大阪市都島区", prefecture=tokyo)
        assert 0 == len(cities)

    def test_addresses(self):
        tokyo = self.postman.search_prefectures("東京", "kanji")[0]
        cities = self.postman.search_cities("新宿", prefecture=tokyo)
        assert 1 == len(cities)
        city = cities[0]

        addresses = self.postman.addresses(city)
        assert 696 == len(addresses)
        assert city == addresses[0].city

    def test_search_addresses(self):
        tokyo = self.postman.search_prefectures("東京", "kanji")[0]
        osaka = self.postman.search_prefectures("大阪", "kanji")[0]
        cities = self.postman.search_cities("新宿", prefecture=tokyo)
        assert 1 == len(cities)
        city = cities[0]

        addresses = self.postman.search_addresses("歌舞伎町", city=city)
        assert 1 == len(addresses)
        address = addresses[0]
        assert "歌舞伎町" == address.kanji
        assert "新宿区" == address.city.kanji
        assert "東京都" == address.prefecture.kanji
        assert "東京都　新宿区　歌舞伎町" == address.concat_kanji
        assert "トウキョウト　シンジュクク　カブキチョウ" == address.concat_kana
        assert "Kabukicho, Shinjuku Ku, Tokyo To" == address.concat_eng

        with pytest.raises(ValueError):
            self.postman.search_addresses("歌舞伎町", city=city, prefecture=osaka)
        addresses = self.postman.search_addresses("歌舞伎町", prefecture=osaka)
        assert 0 == len(addresses)

    def test_get_by_id(self):
        assert isinstance(self.postman.fetch_by_id(0), Prefecture)
        assert isinstance(self.postman.fetch_by_id(1), City)
        assert isinstance(self.postman.fetch_by_id(2), Address)
        assert isinstance(self.postman.fetch_by_id(3), Prefecture)

    def test_prefecture_by_id(self):
        prefecture = self.postman.prefecture_by_id(0)
        assert isinstance(prefecture, Prefecture)

        with pytest.warns(RuntimeWarning):
            self.postman.prefecture_by_id(1)

    def test_city_by_id(self):
        city = self.postman.city_by_id(1)
        assert isinstance(city, City)

        with pytest.warns(RuntimeWarning):
            self.postman.city_by_id(2)

    def test_address_by_id(self):
        address = self.postman.address_by_id(2)
        assert isinstance(address, Address)

        with pytest.warns(RuntimeWarning):
            self.postman.address_by_id(3)


if __name__ == "__main__":
    pytest.main()



import responder
from jusho import Jusho

api = responder.API()
postman = Jusho()

katakana_chart = "ァアィイゥウェエォオカガキギクグケゲコゴサザシジスズセゼソゾタダチヂッツヅテデトドナニヌネノハバパヒビピフブプヘベペホボポマミムメモャヤュユョヨラリルレロヮワヰヱヲンヴヵヶヽヾ"
hiragana_chart = "ぁあぃいぅうぇえぉおかがきぎくぐけげこごさざしじすずせぜそぞただちぢっつづてでとどなにぬねのはばぱひびぴふぶぷへべぺほぼぽまみむめもゃやゅゆょよらりるれろゎわゐゑをんゔゕゖゝゞ"
hir2kat = str.maketrans(hiragana_chart, katakana_chart)
kat2hir = str.maketrans(katakana_chart, hiragana_chart)


@api.route("/")
def index(req, resp):
    resp.html = api.template("index.html")


@api.route("/api/pref")
async def get_pref(req, resp):
    user_input = (await req.media(format="json"))["pref"]
    user_input = user_input.translate(hir2kat).upper()
    prefs = set()
    for type_ in ["kanji", "kana", "eng"]:
        data = postman.search_prefectures(user_input, type_)
        if data:
            prefs.update(data)
    resp.media = {"similar": [{"name": p.kanji, "id": p.id} for p in prefs]}


@api.route("/api/city")
async def get_city(req, resp):
    data = await req.media(format="json")
    pref_id, user_city = data["pref"], data["city"]
    pref = postman.prefecture_by_id(int(pref_id))
    user_city = user_city.translate(hir2kat).upper()
    cities = set()
    for type_ in ["kanji", "kana", "eng"]:
        data = postman.search_cities(user_city, pref, type_)
        if data:
            cities.update(data)
    resp.media = {"similar": [{"name": c.kanji, "id": c.id} for c in cities]}


@api.route("/api/town")
async def get_town(req, resp):
    data = await req.media(format="json")
    city_id, user_town = data["city"], data["town"]
    city = postman.city_by_id(int(city_id))
    user_town = user_town.translate(hir2kat).upper()

    towns = set()
    for type_ in ["kanji", "kana", "eng"]:
        data = postman.search_addresses(user_town, city=city, type_=type_)
        if data:
            towns.update(data)
    resp.media = {"similar": [{"name": t.kanji, "id": t.id} for t in towns]}


@api.route("/api/zip")
async def get_zip(req, resp):
    data = await req.media(format="json")
    town_id = data["town"]
    town = postman.address_by_id(int(town_id))
    resp.media = {"zip": town.hyphen_zip}


if __name__ == "__main__":
    api.run()



from typing import List

from ..models import Prefecture, City, Address


def data_to_city(data: List[str]) -> City:
    pref_length = len(Prefecture.__annotations__)
    pref = Prefecture(*data[:pref_length])
    city = City(int(data[pref_length]), pref, *data[pref_length + 2 :])
    return city


def data_to_address(data: List[str]) -> Address:
    pref_length = len(Prefecture.__annotations__)
    city_length = len(City.__annotations__)
    city = data_to_city(data[: pref_length + city_length])
    address = Address(
        int(data[pref_length + city_length]),
        city,
        *data[pref_length + city_length + 2 :],
    )
    return address



__title__ = "jusho"
__description__ = "Easy wrapper for the zip code data of Japan"
__url__ = "https://github.com/nagataaaas/Jusho"
__version_info__ = ("1", "0", "1")
__version__ = ".".join(__version_info__)
__author__ = "Yamato Nagata"
__author_email__ = "chickenwingswillfly@gmail.com"
__maintainer__ = "Yamato Nagata"
__license__ = "MIT"
__copyright__ = "(c) 2022 by Yamato Nagata"



from .__about__ import __version__
from .jusho import Jusho
from .models import Address, Prefecture, City
from .organize_data import create_database
from . import gateway

__all__ = [
    __version__,
    "Jusho",
    "Address",
    "Prefecture",
    "City",
    "create_database",
    "gateway",
]



from sqlite3 import Cursor

from ..models import Prefecture, City, Address

CREATE_TABLES = """
CREATE TABLE prefectures (
    id integer primary key,
    kanji TEXT NOT NULL,
    kana TEXT NOT NULL,
    eng TEXT NOT NULL
);

CREATE TABLE cities (
    id integer primary key,
    prefecture_id integer references prefectures(id),
    kanji text,
    kana text,
    eng text
);
create index prefecture_id_index on cities(prefecture_id);

CREATE TABLE addresses (
    id integer primary key,
    city_id integer references cities(id),
    admin_division_code text,
    old_zip_code text,
    zip_code text,
    kanji text,
    kana text,
    eng text,
    multiple_zip_code integer,
    multiple_address integer,
    has_chome integer,
    multiple_town_area integer
);
create index city_id_index on addresses(city_id);
create index zip_code_index on addresses(zip_code);
"""


def insert_prefecture(cursor: Cursor, prefecture: Prefecture):
    cursor.execute(
        "INSERT INTO prefectures VALUES (?, ?, ?, ?)",
        (prefecture.id, prefecture.kanji, prefecture.kana, prefecture.eng),
    )


def insert_city(cursor: Cursor, city: City):
    cursor.execute(
        "INSERT INTO cities VALUES (?, ?, ?, ?, ?)",
        (city.id, city.prefecture.id, city.kanji, city.kana, city.eng),
    )


def insert_address(cursor: Cursor, address: Address):
    cursor.execute(
        "INSERT INTO addresses VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        (
            address.id,
            address.city.id,
            address.admin_division_code,
            address.old_zip_code,
            address.zip_code,
            address.kanji,
            address.kana,
            address.eng,
            address.multiple_zip_code,
            address.multiple_address,
            address.has_chome,
            address.multiple_town_area,
        ),
    )



from sqlite3 import Cursor
from typing import List, Optional

from ..models import Prefecture


def fetch_prefectures(cursor: Cursor) -> List[Prefecture]:
    cursor.execute("SELECT * FROM prefectures")
    return [Prefecture(*v) for v in cursor.fetchall()]


def fetch_prefecture_by_id(cursor: Cursor, id_: int) -> Optional[Prefecture]:
    cursor.execute("SELECT * FROM prefectures WHERE id=?", (id_,))
    data = cursor.fetchone()
    return Prefecture(*data) if data else None


def search_prefectures(cursor: Cursor, query: str, type_: str) -> List[Prefecture]:
    cursor.execute(f"SELECT * FROM prefectures WHERE {type_} LIKE '%{query}%'")
    return [Prefecture(*v) for v in cursor.fetchall()]
